{{/*
  Common labels
*/}}
{{- define "aurora.labels" -}}
{{ include "aurora-utils.labels" . }}
{{- end -}}

{{/*
  Selector labels
*/}}
{{- define "aurora.selectorLabels" -}}
{{ include "aurora-utils.selectorLabels" . }}
{{- end -}}

{{/*
Job create extensions name
 */}}
{{- define "aurora.jobCreateExtensionsName" -}}
{{ include "aurora-utils.namingPrefix" . }}-create-extensions
{{- end -}}

{{/*
  The primary secret name for the Database
*/}}
{{- define "aurora.dbPrimaryCredentialsSecretName" -}}
  {{ include "aurora-utils.namingPrefix" . }}-pg-primary-pass
{{- end -}}

{{/*
  Check versions for matching the public Dockerhub images see: https://hub.docker.com/_/postgres/tags

  More recent builds of some of the Postgres images have diverged from the "{{major}}.{{minor}}"
  tag standard. Some (10 through 15) have "{{major}}" but not all. But all seem to have the two
  main OS builds: Debian Bullseye (larger) and Alpine.

  In the newer releases (>=12), they have continued the major/minor tagging format.
  Those version-specific tags are usually consistent with the size of the Bullseye
  builds. For those builds, we can just assume that the major/minor suffix will exist.
  However, for 10.x builds where "-alpine" is the common standard we have to accommodate.
  So this function will treat older major release in a way that reflects the observed
  standards but otherwise expect major/minor tags to be available. However, we're also
  going to add the ability to override this value since none of this has a direct correlation
  to the AWS framework.
*/}}
{{- define "aurora.getLocalVersion" -}}
{{- if (default nil .Values.localVersion) -}}
{{- .Values.localVersion -}}
{{- else -}}
{{- if semverCompare "<= 10.21" .Values.version -}}
{{- .Values.version -}}
{{- else if semverCompare "> 10.21 < 11" .Values.version -}}
{{- .Values.version -}}-alpine
{{- else if semverCompare ">= 11 <= 11.16" .Values.version -}}
{{- .Values.version -}}
{{- else if semverCompare "> 11.16 < 12" .Values.version -}}
{{- .Values.version -}}-bullseye
{{- else -}}
{{- .Values.version -}}
{{- end -}}
{{- end -}}
{{- end -}}


{{/*
  The user password secret name for the Database
*/}}
{{- define "aurora._dbUserPassCredentialsSecretName" -}}
  {{ include "aurora-utils.namingPrefix" . }}-pg-pass
{{- end -}}

{{/*
  The user username secret name for the Database
*/}}
{{- define "aurora._dbUserUsernameCredentialsSecretName" -}}
  {{ include "aurora-utils.namingPrefix" . }}-pg-user
{{- end -}}

{{/*
  Override for db user creation
*/}}
{{- define "aurora.dbOverrideUserList" -}}
{{- if .Values.dbOverrideUserList -}}
dbOverrideUserList: {{ .Values.dbOverrideUserList | quote }}
{{- else -}}
dbUserCredsSecretPassName: {{ include "aurora._dbUserPassCredentialsSecretName" . }}
dbUserCredsSecretUserName: {{ include "aurora._dbUserUsernameCredentialsSecretName" . }}
{{- end -}}
{{- end -}}

{{/*
  Form the db extensions list
*/}}
{{- define "aurora.dbExtensionsList" -}}
{{- if .Values.extensionList -}}
dbExtensionsList: {{ join " " .Values.extensionList | quote }}
{{- end -}}
{{- end -}}

{{/*
  The kubernetes service name for the db endpoint
*/}}
{{- define "aurora.clusterEndpoint" -}}
  {{ include "aurora.clusterEndpointShortName" . }}.{{ include "aurora-utils.appsNamespace" . }}.svc.cluster.local
{{- end -}}

{{/*
  The kubernetes sort service name for the db endpoint
*/}}
{{- define "aurora.clusterEndpointShortName" -}}
  {{ include "aurora-utils.namingPrefix" . }}-endpoint
{{- end -}}

{{/*
  The kubernetes service name for the db endpoint
*/}}
{{- define "aurora.clusterReaderEndpoint" -}}
  {{ include "aurora.clusterReaderEndpointShortName" . }}.{{ include "aurora-utils.appsNamespace" . }}.svc.cluster.local
{{- end -}}

{{/*
  The kubernetes sort service name for the db endpoint
*/}}
{{- define "aurora.clusterReaderEndpointShortName" -}}
  {{ include "aurora-utils.namingPrefix" . }}-reader-endpoint
{{- end -}}

{{/*
  Local Deployment Resource Settings
*/}}
{{- define "aurora.localDeploymentResourceValues" -}}
requests:
  cpu: {{ .Values.localDeployment.resources.requests.cpu }}
  memory: {{ .Values.localDeployment.resources.requests.memory }}
limits:
  memory: {{ .Values.localDeployment.resources.limits.memory }}
{{- end -}}

{{/*
  Check if the aurora port is valid
*/}}
{{- define "aurora.isPortValid" -}}
{{- if or ( eq .Values.port 5432.0 ) ( eq .Values.port 3306.0 ) }}
{{- printf "true" }}
{{ else }}
{{- printf "false" }}
{{- end }}
{{- end -}}

{{/*
  Overrides for cluster db parameter group name
*/}}
{{- define "aurora.clusterParameterGroupName" -}}
{{- if .Values.clusterParameterGroupName -}}
cluster_parameter_group_name = {{ .Values.clusterParameterGroupName | quote }}
{{- end -}}
{{- end -}}

{{/*
  Overrides for cluster db parameters
*/}}
{{- define "aurora.clusterParameterMapping" -}}
{{- if .Values.clusterParameterMapping -}}
cluster_parameter_mapping = {{ .Values.clusterParameterMapping | toJson }}
{{- end -}}
{{- end -}}

{{/*
  Overrides for instance db parameter group name
*/}}
{{- define "aurora.instanceParameterGroupName" -}}
{{- if .Values.instanceParameterGroupName -}}
instance_parameter_group_name = {{ .Values.instanceParameterGroupName | quote }}
{{- end -}}
{{- end -}}

{{/*
  Overrides for instance db parameters
*/}}
{{- define "aurora.instanceParameterMapping" -}}
{{- if .Values.instanceParameterMapping -}}
instance_parameter_mapping = {{ .Values.instanceParameterMapping | toJson }}
{{- end -}}
{{- end -}}

{{/*
    When deployed on AWS, this will emit the necessary volumeMount element definition
    to mount the RDS CA
    Example usage:
    volumeMounts:
    {{- include "rds-ca-volumeMount" . | nindent 10 }}
*/}}
{{- define "rds-ca-volumeMount" -}}
{{- if .Values.global.awsDeployment -}}
{{- if or (ne .Values.global.envCategory "partial") (eq .Values.partialOverrides.installInfra true) -}}
- name: rds-combined-ca-bundle
  mountPath: "/app/certs/rds-combined-ca-bundle.pem"
  subPath: "rds-combined-ca-bundle.pem"
  readOnly: true
{{- end -}}
{{- end -}}
{{- end -}}

{{/*
    When deployed on AWS, this will emit the necessary volume element definition to
    mount the RDS CA
    Example usage:
        volumes:
        {{- include "rds-ca-volume" . | nindent 8 }}
*/}}
{{- define "rds-ca-volume" -}}
{{- if .Values.global.awsDeployment -}}
{{- if or (ne .Values.global.envCategory "partial") (eq .Values.partialOverrides.installInfra true) -}}
- name: rds-combined-ca-bundle
  configMap:
    name: rds-combined-ca-bundle
    items:
    - key: rds-combined-ca-bundle.pem
      path: rds-combined-ca-bundle.pem
{{- end -}}
{{- end -}}
{{- end -}}

{{/*
    When deployed on AWS, this will emit the necessary environment definition to verify-ssl
    and tell psql about the root cert
    Example usage:
        volumes:
        {{- include "rds-ssl-verify-env" . | nindent 12 }}
*/}}
{{- define "rds-ssl-verify-env" -}}
{{- if .Values.global.awsDeployment -}}
{{- if or (ne .Values.global.envCategory "partial") (eq .Values.partialOverrides.installInfra true) -}}
- name: PGSSLMODE
  value: "verify-ca"
- name: PGSSLROOTCERT
  value: "/app/certs/rds-combined-ca-bundle.pem"
{{- end -}}
{{- end -}}
{{- end -}}

{{/*
  Defines the port to be used for connecting to the database
*/}}
{{- define "aurora.port" -}}
{{- if .Values.existingInfraOverrides.enable -}}
{{ .Values.existingInfraOverrides.port | default 5432 }}
{{- else -}}
{{ .Values.port }}
{{- end -}}
{{- end -}}

{{/*
  Defines the database name to be used
*/}}
{{- define "aurora.databaseName" -}}
{{- if .Values.existingInfraOverrides.enable -}}
{{ required "You must specify the values for '.Values.existingInfraOverrides.databaseName' to be created" .Values.existingInfraOverrides.databaseName }}
{{- else -}}
{{ required "You must specify the 'databaseName' to be created" .Values.databaseName }}
{{- end -}}
{{- end -}}

{{/*
  Defines the endpoint configmap params for existing infrastructure
*/}}
{{- define "aurora.endpointCMParams" -}}
{{- if not ( or .Values.existingInfraOverrides.clusterEndpoint .Values.existingInfraOverrides.clusterReaderEndpoint ) }}
{{ printf ("\n\nIf you have set \".Values.existingInfraOverrides.enable\" as true then please provide a value for atleast one of the following: \".Values.existingInfraOverrides.clusterEndpoint\" or \".Values.existingInfraOverrides.clusterReaderEndpoint\"") | fail }}
{{- end }}
{{- if .Values.existingInfraOverrides.clusterEndpoint -}}
endpoint: {{ .Values.existingInfraOverrides.clusterEndpoint }}
{{- end }}
{{- if .Values.existingInfraOverrides.clusterReaderEndpoint }}
readerEndpoint: {{ .Values.existingInfraOverrides.clusterReaderEndpoint }}
{{- end }}
{{- if .Values.multiEnvPostgres }}
{{- range $k, $v := .Values.multiEnvPostgres }}
{{- if not ( or $v.clusterEndpoint $v.clusterReaderEndpoint ) }}
{{ printf ("\n\nIf you have set \".Values.multiEnvPostgres\" as true then please provide a value for atleast one of the following: \"clusterEndpoint\" or \"clusterReaderEndpoint for the individual database instances\"") | fail }}
{{- end }}
{{- end }}
multi_db_json: |
  {{ toJson .Values.multiEnvPostgres }}
{{- end }}
{{- end }}

{{/*
  Defines the output params for -outputs config map
*/}}
{{- define "aurora.outputCMParams" -}}
port: {{ include "aurora.port" . | quote }}
databaseName: {{ include "aurora.databaseName" . | quote }}
{{ if .Values.defaultSchema -}}
defaultSchema: "{{ .Values.defaultSchema }}"
{{ end }}
{{- end }}

{{/*
  Defines the dbServiceCluster Endpoint for existing infrastructure
*/}}
{{- define "aurora.dbServiceClusterEndpoint" -}}
{{- if .Values.existingInfraOverrides.enable -}}
{{- if not ( or .Values.existingInfraOverrides.clusterEndpoint .Values.existingInfraOverrides.clusterReaderEndpoint ) }}
{{ printf ("\n\nIf you have set \".Values.existingInfraOverrides.enable\" as true then please provide a value for atleast one of the following: \".Values.existingInfraOverrides.clusterEndpoint\" or \".Values.existingInfraOverrides.clusterReaderEndpoint\"") | fail }}
{{ end }}
dbServiceClusterEndpoint: {{ .Values.existingInfraOverrides.clusterEndpoint }}
dbServiceClusterReaderEndpoint: {{ .Values.existingInfraOverrides.clusterReaderEndpoint }}
{{ else }}
dbServiceClusterEndpoint: {{ include "aurora.clusterEndpoint" . }}
dbServiceClusterReaderEndpoint: {{ include "aurora.clusterReaderEndpoint" . }}
{{ end }}
{{- end -}}

{{/*
  Aurora dbServiceClusterEndpoint from endpoint configMap
*/}}
{{- define "aurora.dbServiceClusterEndpointconfigmap" -}}
{{ include "aurora-utils.namingPrefix" . }}-endpoint
{{- end -}}


{{/*
  Naming convention for job that resets the Aurora user passwords (for re-encryption)
 */}}
{{- define "aurora-utils.jobResetUserPass" -}}
{{ include "aurora-utils.namingPrefix" . }}-reset-user-pass
{{- end -}}

{{- define "aurora.rdsClusterRoleAssociation" -}}
{{- if .Values.rdsClusterRoleAssociation }}
rds_cluster_role_association = {
    {{- range $key, $val := .Values.rdsClusterRoleAssociation }}
    {{ $key }} = "{{ $val }}"
    {{- end }}
}
{{- end -}}
{{- end -}}

{{- define "aurora.pvTags" -}}
tagSpecification_1: owner={{ required "You must specify the 'owner'" .Values.standardTags.owner }}
tagSpecification_2: sguid={{ required "You must specify the 'sguid'" .Values.sguid }}
tagSpecification_3: src_url={{ required "You must specify the 'src_url'" .Values.standardTags.src_url }}
tagSpecification_4: stage={{ .Values.global.envCategory | default "dev" }}
tagSpecification_5: sku={{ required "You must specify the 'sku'" .Values.standardTags.sku }}
tagSpecification_6: chart_name=postgres
tagSpecification_7: deployment_model=mcs2
tagSpecification_8: env_name={{ .Values.global.env }}
tagSpecification_9: release_name={{ .Release.Name }}
{{- end -}}

{{- define "aurora.storageClassName" -}}
{{ include "aurora-utils.namingPrefix" . }}-pg-sc
{{- end -}}

{{- define "aurora.pvcName" -}}
{{ include "aurora-utils.namingPrefix" . }}-pg-pvc
{{- end -}}

{{- define "aurora.dbStatefulSetName" -}}
{{ include "aurora-utils.namingPrefix" . }}-pg
{{- end -}}

{{- define "aurora.dbStatefulSvcName" -}}
{{ include "aurora-utils.namingPrefix" . }}-pg-svc
{{- end -}}

{{- define "aurora.dbStatefulConfigMapName" -}}
{{ include "aurora-utils.namingPrefix" . }}-endpoint
{{- end -}}
